﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class CollisionScript : MonoBehaviour {
	private int waveNumber;
	public float distanceX, distanceZ;
	public float[] waveAmplitude;
    public Vector2[] impactPos;
    public float[] distance;
    public Color[] colors;
    float[] timers = new float[8];
    float[] alphatimers = new float[8];
    Vector3[] colliders = new Vector3[8];
   // public float speedWaveSpread;
    public float startingAmplitude = 3f;
    public float waveTime = 2f;
    public AnimationCurve fadeCurve;
    public float fadeTime = 0.25f;
    public float spreadSpeeds = 16f;
    
    Mesh mesh;
	// Use this for initialization
	void Start () {
		mesh = GetComponent<MeshFilter>().mesh;
	}
    float[] camOffset = new float[3];
    Color tempCol, tempPrevCol;
    // Update is called once per frame
    void Update () {
	
		for (int i=0; i<8; i++)
        {

			waveAmplitude[i] = GetComponent<Renderer>().material.GetFloat("_WaveAmplitude" + (i+1));
             if (waveAmplitude[i] > 0)

			{
                timers[i] += Time.deltaTime * 360f;
                
                //waveAmplitude[i] *= 0.98f;
                waveAmplitude[i] -= startingAmplitude * Time.deltaTime / waveTime;
                // Debug.Log(Mathf.Sin(timers[i]));
                distance[i] += spreadSpeeds * Time.deltaTime;
				GetComponent<Renderer>().material.SetFloat("_Distance" + (i+1), distance[i]);
				GetComponent<Renderer>().material.SetFloat("_WaveAmplitude" + (i+1), waveAmplitude[i]);
                colors[i].a = 0.75f;
                GetComponent<Renderer>().material.SetColor("_Color" + (i + 1), colors[i]);  
                GetComponent<Renderer>().material.SetFloat("_Time" + (i + 1), timers[i]);

                // colors move /stay with camera
                GetComponent<Renderer>().material.SetFloat("_xImpact" + (i + 1), colliders[i].x);// - camOffset[0]);
                GetComponent<Renderer>().material.SetFloat("_zImpact" + (i + 1), colliders[i].z);// - camOffset[2]);

                distanceX = this.transform.position.x - colliders[i].x;
                distanceZ = this.transform.position.z - colliders[i].z;
                GetComponent<Renderer>().material.SetFloat("_OffsetX" + waveNumber, distanceX / mesh.bounds.size.x * 20f/3f);
                GetComponent<Renderer>().material.SetFloat("_OffsetZ" + waveNumber, distanceZ / mesh.bounds.size.z * 20f / 3f);
            }
            if (waveAmplitude[i] < 0.01)
			{
                GetComponent<Renderer>().material.SetFloat("_WaveAmplitude" + (i+1), 0);
				distance[i] = 0;
                alphatimers[i] += Time.deltaTime / fadeTime;
                
                colors[i].a = fadeCurve.Evaluate(alphatimers[i]) * 0.75f;
                GetComponent<Renderer>().material.SetColor("_Color" + (i + 1), colors[i]);

                if (colors[i].a <= 0)
                {
                }

            }

        }
        camOffset[0] = Camera.main.transform.position.x;
        camOffset[2] = Camera.main.transform.position.z;
        GetComponent<Renderer>().material.SetFloatArray("_CamOffset", camOffset);



        for (int i = nearbyRends.Count-1; i >= 0; i--)
        {
            if (nearbyRendsTimers[i] < 0)
            {
                nearbyRendsTimers[i] += Time.deltaTime *2f;
                nearbyRendsTimers[i] = Mathf.Min(nearbyRendsTimers[i], 0);
                tempCol = nearbyRends[i].material.color;
                tempCol.a = fadeCurve.Evaluate(-nearbyRendsTimers[i]);
                nearbyRends[i].material.color = tempCol;
            }
            if (nearbyRendsTimers[i] > 0)
            {
                nearbyRendsTimers[i] -= Time.deltaTime * 2f;
                tempCol = nearbyRends[i].material.color;
                tempCol.a = fadeCurve.Evaluate(1f-nearbyRendsTimers[i]);
                nearbyRends[i].material.color = tempCol;
                if (nearbyRendsTimers[i] < 0)
                {
                    nearbyRends.RemoveAt(i);
                    nearbyRendsTimers.RemoveAt(i);
                }
            }
        }
        tempCol = Color.clear;
        float distanceAlphaFactor = 1f;
        for (int i = zones.Count - 1; i >= 0; i--)
        {
            distanceAlphaFactor = Vector3.Distance(zones[i].transform.position, this.transform.position) / zones[i].collider.radius;
            distanceAlphaFactor *= 0.6f;

            if (zoneTimers[i] < 0) // fade in
            {
                zoneTimers[i] += Time.deltaTime / 2f;
                zoneTimers[i] = Mathf.Min(zoneTimers[i], 0);
                tempCol += zones[i].zoneColor * (1 - distanceAlphaFactor) / zones.Count * fadeCurve.Evaluate(-zoneTimers[i]);
            }


            else if (zoneTimers[i] > 0) // fade out
            {
                zoneTimers[i] -= Time.deltaTime / 2f;
                if (zoneTimers[i] < 0)
                {
                    zones.RemoveAt(i);
                    zoneTimers.RemoveAt(i);
                }
                else tempCol += zones[i].zoneColor * (1 - distanceAlphaFactor) / zones.Count * fadeCurve.Evaluate(1f- zoneTimers[i]);
            }
            else tempCol += zones[i].zoneColor * (1 - distanceAlphaFactor) / zones.Count;
        }


        tempCol += (1 - tempCol.a) * Color.white;
        tempCol = Color.Lerp(tempPrevCol, tempCol, 0.95f);
        GetComponent<Renderer>().material.SetColor("_Color", tempCol);
        tempPrevCol = tempCol;
    }
    List<Renderer> nearbyRends = new List<Renderer>();
    List<float> nearbyRendsTimers = new List<float>();
    void OnCollisionEnter(Collision col){
		if (col.rigidbody)
		{
            Addimpact(col.transform.position);
           
		}
      /*  if (col.gameObject.GetComponent<Renderer>())
        {
            nearbyRends.Add(col.gameObject.GetComponent<Renderer>());
            nearbyRendsTimers.Add(-1f);
        }*/

	}/*
    void OnCollisionExit(Collision col)
    {
        if (col.gameObject.GetComponent<Renderer>())
        {
            int index = nearbyRends.IndexOf(col.gameObject.GetComponent<Renderer>());
            nearbyRendsTimers[index] = 1f;
        }

    }*/
    List<ZoneTrigger> zones = new List<ZoneTrigger>();
    List<float> zoneTimers = new List<float>();
    public void EnterZone (ZoneTrigger zone)
    {
        zones.Add(zone);
        zoneTimers.Add(-1f);
    }
    public void ExitZone(ZoneTrigger zone)
    {
        //zones.Remove(zone);
        int index = zones.IndexOf(zone);
        if (zoneTimers[index] == 0f)
            zoneTimers[index] = 1f;
        else zoneTimers[index] = -zoneTimers[index]; // reverse fade
    }
    public void Addimpact(Vector3 col)
    {
        waveNumber++;
        if (waveNumber == 9)
        {
            waveNumber = 1;
        }
        waveAmplitude[waveNumber - 1] = 0;
        distance[waveNumber - 1] = 0;
        timers[waveNumber - 1] = 0;
        colliders[waveNumber - 1] = col;
        alphatimers[waveNumber - 1] = 0;

        distanceX = this.transform.position.x - col.x;
        distanceZ = this.transform.position.z - col.y;
        impactPos[waveNumber - 1].x = col.x;
        impactPos[waveNumber - 1].y = col.y;

        GetComponent<Renderer>().material.SetFloat("_xImpact" + waveNumber, col.x);
        GetComponent<Renderer>().material.SetFloat("_zImpact" + waveNumber, col.y);

        GetComponent<Renderer>().material.SetFloat("_OffsetX" + waveNumber, distanceX / mesh.bounds.size.x * 20f / 3f);
        GetComponent<Renderer>().material.SetFloat("_OffsetZ" + waveNumber, distanceZ / mesh.bounds.size.z * 20f / 3f);

        GetComponent<Renderer>().material.SetFloat("_WaveAmplitude" + waveNumber, startingAmplitude);
    }
}
